
#define DWORD unsigned long
#define WORD short

typedef struct {
	char 	cRIFF[4];		   /* "RIFF" */
	DWORD 	nFileLength;       /* data buffer length + 36 */
	char 	cWAVEfmt[8];       /* "WAVEfmt " */
    DWORD 	wSize;			   /* 16 */
    WORD    wFormatTag;        /* format type */
    WORD    nChannels;         /* number of channels (i.e. mono, stereo, etc.) */
    DWORD   nSamplesPerSec;    /* sample rate */
    DWORD   nAvgBytesPerSec;   /* for buffer estimation */
    WORD    nBlockAlign;       /* block size of data */
    WORD    wBitsPerSample;    /* number of bits per sample */
	char	cData[4];		   /* "data" */
	DWORD 	nDataLength;       /* data buffer length */
} CWAVEFORMAT;

#define BOOL 	int
#define TRUE	1
#define FALSE	0

typedef unsigned int CMIRESULT;
#define CMI_NOERROR			0 
#define CMI_NOPCIBIOS		1 // No PCI BIOS interface present.
#define CMI_NOAUDIOCHIP		2 // Can not find out the CMI8338 or CMI8738 PCI audio chip.
#define CMI_IRQNOTSUPPORT	3 // This Interrupt Request Line is not supported.
#define CMI_CANNOTALLOCATE	4 // Memory allocation error.


// MP3 iformation
enum e_version { MPEG2_LSF, MPEG1 };
enum e_sample_frequency { fourtyfour_point_one, fourtyeight, thirtytwo };
enum e_mode { stereo, joint_stereo, dual_channel, single_channel };
enum e_emphasis { none, ms50_15, reserved, CCITTJ17 };

typedef struct 
{
	char TAG[3];
	char MP3Title[30];
	char MP3Artist[30];
	char MP3Album[30];
	char MP3Year[4];
	char MP3Comment[30];
	unsigned char MP3Genre;
} ID3Info;

typedef struct {
	e_mode h_mode;
	short h_protection_bit, h_layer, h_padding_bit;
	DWORD h_bitrate;
	e_version h_version;
	DWORD h_sample_frequency;
	BOOL h_copyright, h_original, h_private;
	e_emphasis h_emphasis;
	ID3Info ID3Tag;
} MP3FORMAT;
	
// Chip information
typedef struct {
	int VenderID, DeviceID, IOBaseAddress, IRQ, IRQInterrupt;
} CHIPINFO;



// ----------------------------------------------------------------------------------------
// Initialization
CMIRESULT InitialCMedia(CHIPINFO &ChipInformation);

// Exit
void ExitCMedia();

// Get Wave and MP3 file information. The file will be used to playback.
BOOL GetWaveFileInformation(FILE *SourceFile, CWAVEFORMAT &PCMFormat);
BOOL GetMP3FileInformation(FILE *SourceFile, CWAVEFORMAT &PCMFormat, MP3FORMAT &MP3Format);

// Playback control functions. Before playback, you should do initialization and get file information.
void Playback();
void Pause();
void StopPlayback(); 
BOOL Forward();
BOOL Backward();
void FillPlaybackData();

// Sound Effect
void SetMP3EqualizerBand(float EQBandRatio[2][32]);

// Playback information
BOOL IsPlaybackEnd();
DWORD GetPlaybackLength();
void GetPeakmeter(unsigned int &LeftChannelPeakmeter, unsigned int &RightChannelPeakmeter); // 0 ~ 100

// Audio chip Mixer control functions
void SetMasterVolume(unsigned int Volume);   //The range of Master Volume is 0 ~ 15
unsigned int GetMasterVolume();
void EnableSPDIFOut(BOOL bEnable);
void EnableRearSpeaker(BOOL bEnable); 

